@@ -6,7 +6,7 @@
  *      SPARC AUDIO .AU-files
  *      Raw Data
  *
- *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
+ *  Copyright (c) by Jaroslav Kysela <perex@perex.cz>
  *  Based on vplay program by Michael Beck
  *
  *
@@ -37,6 +37,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <limits.h>
+#include <time.h>
 #include <locale.h>
 #include <alsa/asoundlib.h>
 #include <assert.h>
@@ -302,7 +303,7 @@ static void pcm_list(void)
 
 static void version(void)
 {
-	printf("%s: version " SND_UTIL_VERSION_STR " by Jaroslav Kysela <perex@suse.cz>\n", command);
+	printf("%s: version " SND_UTIL_VERSION_STR " by Jaroslav Kysela <perex@perex.cz>\n", command);
 }
 
 static void signal_handler(int sig)
@@ -322,7 +323,7 @@ static void signal_handler(int sig)
 		close(fd);
 		fd = -1;
 	}
-	if (handle) {
+	if (handle && sig != SIGABRT) {
 		snd_pcm_close(handle);
 		handle = NULL;
 	}
@@ -1215,10 +1216,10 @@ static void compute_max_peak(u_char *data, size_t count)
 		putchar('\r');
 		for (val = 0; val <= perc / 2 && val < 50; val++)
 			putchar('#');
-		for (; val < maxperc / 2 && val < 50; val++)
+		for (; val <= maxperc / 2 && val < 50; val++)
 			putchar(' ');
 		putchar('+');
-		for (++val; val < 50; val++)
+		for (++val; val <= 50; val++)
 			putchar(' ');
 
 		printf("| %02i%%", maxperc);
